@@ -309,19 +309,9 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
                   else x %*% start)
               }
               else family$linkfun(mustart)
- 
-
-   ##mu.eta <- family$mu.eta
-   ##Dd <- family$Dd
-
- 
+  
    mu <- linkinv(eta);etaold <- eta
      
-   ## need an initial `null deviance' to test for initial divergence...
-   ## if (!is.null(start)) null.coef <- start - can be on edge of feasible - not good
-   #null.eta <- as.numeric(x%*%null.coef + as.numeric(offset))
-   #old.pdev <- sum(dev.resids(y, linkinv(null.eta), weights,theta)) + t(null.coef)%*%St%*%null.coef 
-
    coefold <- null.coef
    conv <-  boundary <- FALSE
  
@@ -341,16 +331,7 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
         good <- is.finite(w)&is.finite(wz)
         z[!is.finite(z)] <- 0 ## avoid NaN in .C call - unused anyway
       } else use.wy <- family$use.wz
-      
-      #if (sum(!good)) {
-      #  good1 <- is.finite(w)&good ## make sure w finite too
-      #  w[!is.finite(w)] <- 0      ## clear infinite w
-      #  w[!good1&w==0] <- max(w)*.Machine$double.eps^.5 ## reset zero value weights for problem elements
-      #  dd$Deta.Deta2[!good] <- .5*dd$Deta[!good]/w[!good] ## reset problem elements to finite
-      #  good <- is.finite(dd$Deta.Deta2) ## check in case Deta not finite, for example
-      #}
-      #z <- (eta-offset)[good] - dd$Deta.Deta2[good] ## - .5 * dd$Deta[good] / w
-      
+          
       oo <- .C(C_pls_fit1,   
                y=as.double(z[good]),X=as.double(x[good,]),w=as.double(w[good]),wy = as.double(wz[good]),
                      E=as.double(Sr),Es=as.double(Eb),n=as.integer(sum(good)),
@@ -364,7 +345,6 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
         ## index weights that are finite and positive 
         good <- is.finite(dd$Deta2)
         good[good] <- dd$Deta2[good]>0 
-        #w <- dd$Deta2*.5; 
         w[!good] <- 0
         wz <- w*(eta-offset) - .5*dd$Deta
         z <- (eta-offset) - dd$Deta.Deta2
@@ -374,11 +354,7 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
           good <- is.finite(w)&is.finite(wz)
           z[!is.finite(z)] <- 0 ## avoid NaN in .C call - unused anyway
         } else use.wy <- family$use.wz
-        #thresh <- max(w[good])*.Machine$double.eps^.5
-        #w[w < thresh] <- thresh
-        #good <- is.finite(dd$Deta)
-        #z <- (eta-offset)[good] - .5 * dd$Deta[good] / w[good]
-       
+      
         oo <- .C(C_pls_fit1, ##C_pls_fit1,
                   y=as.double(z[good]),X=as.double(x[good,]),w=as.double(w[good]),wy = as.double(wz[good]),
                      E=as.double(Sr),Es=as.double(Eb),n=as.integer(sum(good)),
@@ -485,7 +461,6 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
        ## drop same parameter every iteration!)       
        grad <- 2 * t(x[good,])%*%((w[good]*(x%*%start)[good]-wz[good]))+ 2*St%*%start 
        if (max(abs(grad)) > control$epsilon*max(abs(start+coefold))/2) {
-      ## if (max(abs(start-coefold))>control$epsilon*max(abs(start+coefold))/2) {
          old.pdev <- pdev  ## not converged quite enough
          coef <- coefold <- start
          etaold <- eta 
@@ -516,32 +491,17 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
    dd <- dDeta(y,mu,weights,theta,family,deriv)
    w <- dd$Deta2 * .5
    z <- (eta-offset) - dd$Deta.Deta2 ## - .5 * dd$Deta[good] / w
-   wf <- dd$EDeta2 * .5 ## Fisher type weights 
+   wf <- pmax(0,dd$EDeta2 * .5) ## Fisher type weights 
    wz <- w*(eta-offset) - 0.5*dd$Deta ## Wz finite when w==0
   
    gdi.type <- if (any(abs(w)<.Machine$double.xmin*1e20)||any(!is.finite(z))) 1 else 0   
    good <- is.finite(wz)&is.finite(w)   
       
-
-   ## exclude points for which gradient and second deriv are effectively zero and 
-   ## points with non finite second deriv or deriv ratio... 
-   #min.Deta <- mean(abs(dd$Deta[is.finite(dd$Deta)]))*.Machine$double.eps*.001
-   #min.Deta2 <- mean(abs(dd$Deta2[is.finite(dd$Deta2)]))*.Machine$double.eps*.001
-   #good <- is.finite(dd$Deta.Deta2)&is.finite(w)&!(abs(dd$Deta2) < min.Deta2 & abs(dd$Deta) < min.Deta) 
-   #if (control$trace&sum(!good)>0) cat("\n",sum(!good)," not good\n")
-   #w <- w[good] 
-
-   #z <- (eta-offset)[good] - dd$Deta.Deta2[good] ## - .5 * dd$Deta[good] / w
-   #wf <- dd$EDeta2[good] * .5 ## Fisher type weights 
-   #wz <- w*(eta-offset)[good] - 0.5*dd$Deta[good]
-
-   #residuals <- rep.int(NA, nobs)
    residuals <- z - (eta - offset)
    residuals[!is.finite(residuals)] <- NA 
    z[!is.finite(z)] <- 0 ## avoid passing NA etc to C code  
 
    ntot <- length(theta) + length(sp)
-   ## if (deriv>1) n2d <- ntot*(1+ntot)/2 else n2d <- 0 
    rSncol <- unlist(lapply(UrS,ncol))
    ## Now drop any elements of dd that have been dropped in fitting...
    if (sum(!good)>0) { ## drop !good from fields of dd, weights and pseudodata
@@ -570,10 +530,7 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
        } 
      }
    }
-   ## can't have zero weights in gdi2 call - superceded by type=1 handling of w==0
-   #mwb <- max(abs(w))*.Machine$double.eps
-   #mwa <- min(abs(w[w!=0]))*.0001; if (mwa==0) mwa <- mwb
-   #w[w==0] <- min(mwa,mwb);
+
    oo <- .C(C_gdi2,
             X=as.double(x[good,]),E=as.double(Sr),Es=as.double(Eb),rS=as.double(unlist(rS)),
             U1 = as.double(U1),sp=as.double(exp(sp)),theta=as.double(theta),
@@ -582,7 +539,7 @@ gam.fit4 <- function(x, y, sp, Eb,UrS=list(),
             Det2=as.double(dd$Deta2),Dth2=as.double(dd$Dth2),Det.th=as.double(dd$Detath),
             Det2.th=as.double(dd$Deta2th),Det3=as.double(dd$Deta3),Det.th2 = as.double(dd$Detath2),
             Det4 = as.double(dd$Deta4),Det3.th=as.double(dd$Deta3th), Deta2.th2=as.double(dd$Deta2th2),
-            beta=as.double(coef),b1=as.double(rep(0,ntot*ncol(x))),w1=rep(0,ntot*length(z)),
+            beta=as.double(coef),b1=as.double(rep(0,ntot*ncol(x))),w1=as.double(rep(0,ntot*length(z))),
             D1=as.double(rep(0,ntot)),D2=as.double(rep(0,ntot^2)),
             P=as.double(0),P1=as.double(rep(0,ntot)),P2 = as.double(rep(0,ntot^2)),
             ldet=as.double(1-2*(scoreType=="ML")),ldet1 = as.double(rep(0,ntot)), 
@@ -845,7 +802,7 @@ gam.fit5 <- function(x,y,lsp,Sl,weights=NULL,offset=NULL,deriv=2,family,
     ll <- llf(y,x,coef1,weights,family,offset=offset,deriv=1) 
     ll1 <- ll$l - (t(coef1)%*%St%*%coef1)/2
     khalf <- 0;fac <- 2
-    while (ll1 < ll0 && khalf < 25) { ## step halve until it succeeds...
+    while ((!is.finite(ll1)||ll1 < ll0) && khalf < 25) { ## step halve until it succeeds...
       step <- step/fac;coef1 <- coef + step
       ll <- llf(y,x,coef1,weights,family,offset=offset,deriv=0)
       ll1 <- ll$l - (t(coef1)%*%St%*%coef1)/2
@@ -858,12 +815,12 @@ gam.fit5 <- function(x,y,lsp,Sl,weights=NULL,offset=NULL,deriv=2,family,
       if (khalf>5) fac <- 5
     } ## end step halve
  
-    if (ll1 < ll0) { ## switch to steepest descent... 
+    if (!is.finite(ll1) || ll1 < ll0) { ## switch to steepest descent... 
       step <- -.5*drop(grad)*mean(abs(coef))/mean(abs(grad))
       khalf <- 0
     }
 
-    while (ll1 < ll0 && khalf < 25) { ## step cut until it succeeds...
+    while ((!is.finite(ll1)||ll1 < ll0) && khalf < 25) { ## step cut until it succeeds...
       step <- step/10;coef1 <- coef + step
       ll <- llf(y,x,coef1,weights,family,offset=offset,deriv=0)
       ll1 <- ll$l - (t(coef1)%*%St%*%coef1)/2
@@ -875,7 +832,7 @@ gam.fit5 <- function(x,y,lsp,Sl,weights=NULL,offset=NULL,deriv=2,family,
       khalf <- khalf + 1
     }
 
-    if (ll1 >= ll0||iter==control$maxit) { ## step ok. Accept and test
+    if ((is.finite(ll1)&&ll1 >= ll0)||iter==control$maxit) { ## step ok. Accept and test
       coef <- coef + step
       ## convergence test...
       ok <- (iter==control$maxit||(abs(ll1-ll0) < control$epsilon*abs(ll0) 
@@ -1097,7 +1054,7 @@ gam.fit5 <- function(x,y,lsp,Sl,weights=NULL,offset=NULL,deriv=2,family,
     db.drho <- matrix(0,length(bdrop),ncol(d1b));db.drho[!bdrop,] <- d1b
   } else db.drho <- d1b
   ## and undo re-para...
-  if (!is.null(d1b)) db.drho <- t(Sl.repara(rp$p,t(db.drho),inverse=TRUE,both.sides=FALSE)) 
+  if (!is.null(d1b)) db.drho <- t(Sl.repara(rp$rp,t(db.drho),inverse=TRUE,both.sides=FALSE)) 
 
   ret <- list(coefficients=coef,family=family,y=y,prior.weights=weights,
        fitted.values=fitted.values, linear.predictors=linear.predictors,
@@ -1282,34 +1239,51 @@ gam.fit5.post.proc <- function(object,Sl,L,lsp0,S,off) {
     lbb[ibd,ibd] <- lbbt
   }  
 
+  edge.correct <- FALSE 
   ## compute the smoothing parameter uncertainty correction...
-  if (!is.null(object$outer.info$hess)&&!is.null(object$db.drho)) { 
-    if (!is.null(L)) object$db.drho <- object$db.drho%*%L ## transform to derivs w.r.t. working
-    ev <- eigen(object$outer.info$hess,symmetric=TRUE)
-    d <- ev$values;ind <- d <= 0
-    d[ind] <- 0;d[!ind] <- 1/sqrt(d[!ind])
-    Vc <- crossprod((d*t(ev$vectors))%*%t(object$db.drho))
-    #dpv <- rep(0,ncol(object$outer.info$hess));M <- length(off)
-    #dpv[1:M] <- 1/100 ## prior precision (1/var) on log smoothing parameters
-    #Vr <- chol2inv(chol(object$outer.info$hess + diag(dpv,ncol=length(dpv))))[1:M,1:M]
-    #Vc <- object$db.drho%*%Vr%*%t(object$db.drho)
-    
-    #dpv[1:M] <- 1/10 ## prior precision (1/var) on log smoothing parameters
-    #Vr <- chol2inv(chol(object$outer.info$hess + diag(dpv,ncol=length(dpv))))[1:M,1:M]
-    #M <- length(off)
-    d <- ev$values; d[ind] <- 0;
-    d <- d + 1/50 #d[1:M] <- d[1:M] + 1/50 
-    d <- 1/sqrt(d)
-    Vr <- crossprod(d*t(ev$vectors))
-    #Vc2 <- Vb.corr(R,L,S,off,dw=NULL,w=NULL,log(object$sp),Vr)
-
-    Vc <- Vb + Vc #+ Vc2  ## Bayesian cov matrix with sp uncertainty
-    ## reverse the various re-parameterizations...
-  } else Vc <- Vb
-  Vc <- Sl.repara(object$rp,Vc,inverse=TRUE) 
-  Vc <-  Sl.initial.repara(Sl,Vc,inverse=TRUE)
+  if (!is.null(object$outer.info$hess)&&!is.null(object$db.drho)) {
+    hess <- object$outer.info$hess
+    edge.correct <- if (is.null(attr(hess,"edge.correct"))) FALSE else TRUE
+    K <- if (edge.correct) 2 else 1
+    for (k in 1:K) {
+      if (k==1) { ## fitted model computations
+        db.drho <- object$db.drho
+        dw.drho <- object$dw.drho
+        lsp <- log(object$sp)
+      } else { ## edge corrected model computations
+        db.drho <- attr(hess,"db.drho1")
+        dw.drho <- attr(hess,"dw.drho1")
+        lsp <- attr(hess,"lsp1")
+	hess <- attr(hess,"hess1")
+      }
+      if (!is.null(L)) db.drho <- db.drho%*%L ## transform to derivs w.r.t. working
+      ev <- eigen(hess,symmetric=TRUE)
+      d <- ev$values;ind <- d <= 0
+      d[ind] <- 0;d[!ind] <- 1/sqrt(d[!ind])
+      Vc <- crossprod((d*t(ev$vectors))%*%t(db.drho)) ## first correction
+   
+      d <- ev$values; d[ind] <- 0;
+      d <- if (k==1) 1/sqrt(d+1/50) else 1/sqrt(d+1e-7)
+  
+      Vr <- crossprod(d*t(ev$vectors))
+     
+      if (k==1) {
+        Vc1 <- Vc; Vr1 <- Vr; lsp1 <- lsp ## un-shifted version to use for edf
+      } 
+      ## reverse the various re-parameterizations...
+    }
+    rp <- if (edge.correct) attr(object$outer.info$hess,"rp") else object$rp
+    Vc <- Sl.repara(rp,Vc,inverse=TRUE) 
+    Vc <-  Sl.initial.repara(Sl,Vc,inverse=TRUE)
+  } else Vc <- 0
   Vb <- Sl.repara(object$rp,Vb,inverse=TRUE)
   Vb <-  Sl.initial.repara(Sl,Vb,inverse=TRUE)
+  Vc <- Vb + Vc
+  if (edge.correct) { 
+    Vc1 <- Sl.repara(object$rp,Vc1,inverse=TRUE) 
+    Vc1 <-  Sl.initial.repara(Sl,Vc1,inverse=TRUE)
+    Vc1 <- Vb + Vc1 
+  } 
   R <- Sl.repara(object$rp,R,inverse=TRUE,both.sides=FALSE)
   R <-  Sl.initial.repara(Sl,R,inverse=TRUE,both.sides=FALSE,cov=FALSE)
   F <- Vb%*%crossprod(R)
@@ -1322,12 +1296,12 @@ gam.fit5.post.proc <- function(object,Sl,L,lsp0,S,off) {
   ## justification only applies to sum(edf1/2) not elementwise   
   if (!is.null(object$outer.info$hess)&&!is.null(object$db.drho)) { 
     ## second correction term is easier computed in original parameterization...
-    Vc2 <- Vb.corr(R,L,lsp0,S,off,dw=NULL,w=NULL,log(object$sp),Vr)
-    Vc <- Vc + Vc2
+    Vc <- Vc + Vb.corr(R,L,lsp0,S,off,dw=NULL,w=NULL,lsp,Vr)
+    if (edge.correct) Vc1 <- Vc1 +
+      Vb.corr(R,L,lsp0,S,off,dw=NULL,w=NULL,lsp1,Vr1) else Vc1 <- Vc
   }
-  edf1 <- 2*edf - rowSums(t(F)*F)
-  #edf2 <- diag(Vc%*%crossprod(R)) 
-  edf2 <- rowSums(Vc*crossprod(R))
+  edf1 <- 2*edf - rowSums(t(F)*F) 
+  edf2 <- if (edge.correct) rowSums(Vc1*crossprod(R)) else rowSums(Vc*crossprod(R))
   if (sum(edf2)>sum(edf1)) edf2 <- edf1 
   ## note hat not possible here...
   list(Vc=Vc,Vb=Vb,Ve=Ve,edf=edf,edf1=edf1,edf2=edf2,F=F,R=R)
